#########------------Paquetes------------#########
library(dplyr)
library(cluster)
library(klaR)
library(circlize)
library(devtools)
library(ComplexHeatmap)
library(fpc)
library(ggplot2)
library(reshape2)
library(purrr)
library(dendextend)
library(factoextra)
library(Rtsne)
library(ggpubr)
#
#########------------Datos y Matriz------------#########
resultados <- read.table("Deteccion.table.tsv", #nombre del documento
                         sep = "\t", # separado por
                         quote = "", #carcateres limitados por
                         header = T) # encabezado
View(resultados) #ver tabla

matriz <- as.matrix(resultados[,c(3:29)]) # separar los datos de factores de virulencia para matriz de correlación 
rownames(matriz) <- resultados[,1] # Recortar y agregar los nombres de la matriz
str(resultados) #verificar que estén todos los factores
#
#########------------Clusters------------#########
#---------------------Gower Distance-------------#
gower.dist <- daisy((matriz), metric = c("gower"))
#--------------------Distancia euclidiana--------#
euclidean.dist <- dist(matriz, method = "euclidean")
#---------------------Cluster aglomerativo-------#
fit.gower <- hclust(gower.dist, method = "complete") 
fit.euclidian <- hclust(euclidean.dist, method = "complete")
plot(fit.gower)
plot(fit.euclidian)
#########------------Función para evaluar clusters------------#########
cstats.table <- function(dist, tree, k) {
  clust.assess <- c("cluster.number","n","within.cluster.ss","average.within","average.between",
                    "wb.ratio","dunn2","avg.silwidth")
  clust.size <- c("cluster.size")
  stats.names <- c()
  row.clust <- c()
  output.stats <- matrix(ncol = k, nrow = length(clust.assess))
  cluster.sizes <- matrix(ncol = k, nrow = k)
  for(i in c(1:k)){
    row.clust[i] <- paste("Cluster-", i, " size")
  }
  for(i in c(2:k)){
    stats.names[i] <- paste("Test", i-1)
    
    for(j in seq_along(clust.assess)){
      output.stats[j, i] <- unlist(cluster.stats(d = dist, clustering = cutree(tree, k = i))[clust.assess])[j]
      
    }
    
    for(d in 1:k) {
      cluster.sizes[d, i] <- unlist(cluster.stats(d = dist, clustering = cutree(tree, k = i))[clust.size])[d]
      dim(cluster.sizes[d, i]) <- c(length(cluster.sizes[i]), 1)
      cluster.sizes[d, i]
      
    }
  }
  
  output.stats.df <- data.frame(output.stats)
  cluster.sizes <- data.frame(cluster.sizes)
  cluster.sizes[is.na(cluster.sizes)] <- 0
  rows.all <- c(clust.assess, row.clust)
  # rownames(output.stats.df) <- clust.assess
  output <- rbind(output.stats.df, cluster.sizes)[ ,-1]
  colnames(output) <- stats.names[2:k]
  rownames(output) <- rows.all
  is.num <- sapply(output, is.numeric)
  output[is.num] <- lapply(output[is.num], round, 2)
  output
}
#########------------Evaluar clusters------------#########
stats.gower.divisive <- cstats.table(gower.dist, fit.gower, 15)
stats.euclidean.divisive <- cstats.table(euclidean.dist, fit.euclidian, 15)

#--------------------Elbow-----------------------#
ggplot(data = data.frame(t(cstats.table(gower.dist, fit.gower, 15))), 
       aes(x=cluster.number, y=within.cluster.ss)) + 
  geom_point()+
  geom_line()+
  ggtitle("Gower distance complete clustering") +
  labs(x = "Num.of clusters", y = "Within clusters sum of squares (SS)") +
  theme(plot.title = element_text(hjust = 0.5))

ggplot(data = data.frame(t(cstats.table(euclidean.dist, fit.euclidian, 15))), 
       aes(x=cluster.number, y=within.cluster.ss)) + 
  geom_point()+
  geom_line()+
  ggtitle("Euclidean distance complete clustering") +
  labs(x = "Num.of clusters", y = "Within clusters sum of squares (SS)") +
  theme(plot.title = element_text(hjust = 0.5))

#-----------------Silhouette--------------------#
Silhouette.gower <- ggplot(data = data.frame(t(cstats.table(gower.dist, fit.gower, 15))), 
       aes(x=cluster.number, y=avg.silwidth)) + 
  geom_point()+
  geom_line()+
  ggtitle("Silhouette Gower dist - Center cluster") +
  labs(x = "Num.of clusters", y = "Average silhouette width") +
  theme(plot.title = element_text(hjust = 0.5))

Silhouette.euclidean <- ggplot(data = data.frame(t(cstats.table(euclidean.dist, fit.euclidian, 15))), 
       aes(x=cluster.number, y=avg.silwidth)) + 
  geom_point()+
  geom_line()+
  ggtitle("Silhouette Euclidean dist - Center cluster") +
  labs(x = "Num.of clusters", y = "Average silhouette width") +
  theme(plot.title = element_text(hjust = 0.5))
#########------------Gráficar dendogramas------------#########
#-----------------------Gower-------------------#
dendro.gower <- as.dendrogram(fit.gower)

dendro.col.gower <- dendro.gower %>%
  set("branches_k_color", k = 3,
      value =   c("darkslategray", "red", "purple"#,
                  #"gold3", "darkcyan", "cyan3"
                  )) %>%
  set("branches_lwd", 0.6) %>%
  set("labels_colors", 
      value = c("darkslategray")) %>% 
  set("labels_cex", 0.5)

ggd1.gower <- as.ggdend(dendro.col.gower)

ggplot(ggd1.gower, theme = theme_minimal()) +
  labs(x = "Num. observations", y = "Height", title = "Gower dist, Center cluster, Dendrogram, k = 6")

# Radial plot
dendro.gower.plot <- ggplot(ggd1.gower, labels = T) + 
  scale_y_reverse(expand = c(0.2, 0)) +
  coord_polar(theta="x") + ggtitle("Gower dendrogram")

#-----------------------Euclidean-------------------#
dendro.euclidean <- as.dendrogram(fit.euclidian)

dendro.col.euclidean <- dendro.euclidean %>%
  set("branches_k_color", k = 6,
      value =   c("darkslategray", "red", "purple", "gold3", "darkcyan", "cyan3")) %>%
  set("branches_lwd", 0.6) %>%
  set("labels_colors", 
      value = c("darkslategray")) %>% 
  set("labels_cex", 0.5)

ggd1.euclidean <- as.ggdend(dendro.col.euclidean)

ggplot(ggd1.euclidean, theme = theme_minimal()) +
  labs(x = "Num. observations", y = "Height", title = "Gower dist, Center cluster, Dendrogram, k = 6")

# Radial plot 
dendro.euclidean.plot <- ggplot(ggd1.euclidean, labels = T) + 
  scale_y_reverse(expand = c(0.2, 0)) +
  coord_polar(theta="x") + ggtitle("Euclidean dendrogram")


#########------------Extraer muestras por cluster------------#########
gower.groups <- data.frame(cutree(fit.gower,k=6))
euclidean.groups <- data.frame(cutree(fit.euclidian, k = 6))
groups.clusters <- cbind(gower.groups, euclidean.groups)

#########------------Gráficar cluster------------#########

#-------------------Plot Gower---------------------#
tsne_obj <- Rtsne(gower.dist, is_distance = TRUE)

tsne_data <- tsne_obj$Y %>%
  data.frame() %>%
  setNames(c("X", "Y")) %>%
  mutate(cluster = factor(cutree(fit.gower,k=3)),
         name = rownames(matriz))

grupos.gower <- ggplot(aes(x = X, y = Y), data = tsne_data) +
  geom_point(aes(color = cluster), size = 2) + 
  scale_fill_manual(values = c("red", "purple", "yellow", "blue", "pink", "gray")) +
  theme_classic() + ggtitle("Gower dist - Center cluster")

#-------------------Plot euclidean---------------------#
tsne_obj.euclidean <- Rtsne(euclidean.dist, is_distance = TRUE)

tsne_data.euclidean <- tsne_obj.euclidean$Y %>%
  data.frame() %>%
  setNames(c("X", "Y")) %>%
  mutate(cluster = factor(cutree(fit.euclidian,k = 3)),
         name = rownames(matriz))

grupos.euclidean <- ggplot(aes(x = X, y = Y), data = tsne_data.euclidean) +
  geom_point(aes(color = cluster), size = 2) + 
  scale_fill_manual(values = c("red", "purple", "yellow", "blue", "pink", "gray")) + 
  theme_classic() + ggtitle("Euclidian dist - Center cluster")

#

#########------------ggarrenge------------#########
ggarrange(Silhouette.gower, grupos.gower)
ggarrange(Silhouette.euclidean, grupos.euclidean)
dendro.gower.plot
dendro.euclidean.plot

#########------------Heatmap------------#########

#-------------------Colores para el heatmap------------------#
col_fun <- colorRamp2(c(0, 1), c("#EEEFF7", "#FF0A1C")) #escala de color

#-------------------Anotaciones columnas---------------------#
Diagnosis <- resultados[,2] # recortar columna de diagnostico
Diagnosis == "Moderate P"
Diagnosis[74] <- "Moderate P."
Diagnosis <- droplevels(Diagnosis)
Diagnosis <- gsub("Moderate P", "Moderate P.", Diagnosis, ignore.case = T)
Phylo <- resultados[,"PHYLOGROUP"]
Serotipo <- resultados[,"SEROTYPE"]
Biofilm <- resultados[,"BIOFILM.FORMATION"]
Biofilm <- gsub("MODERATE", "Moderate", Biofilm)
Biofilm <- gsub("NON-PRODUCING", "Non-producing", Biofilm)
Biofilm <- gsub("STRONG", "Strong", Biofilm)
Biofilm <- gsub("WEAK", "Weak", Biofilm)

coann <- HeatmapAnnotation(Phylogroup = Phylo,   # introducir las variables
                           Diagnosis = Diagnosis,
                           Biofilm = Biofilm,
                           col = list(Phylogroup = c("A"="#0455BF", "B1"="#A4E1EB", # colores para los componentes de las anotaciones
                                                     "B2"="orange","D"="brown3",
                                                     "E"="#F89BC8", "F"="darkorchid2"),
                                      "Diagnosis"= c("Chronic P."="yellow",
                                                     "Gingivitis"="red",
                                                     "Moderate P." = "lightgreen"),
                                      "Biofilm" = c("Moderate" = "brown2",
                                                    "Non-producing" = "white",
                                                    "Strong" = "brown",
                                                    "Weak" = "burlywood1")),
                           annotation_height = unit(c(5,7,2,2,10,8), "mm"), gp = gpar(col = "white"),
                           annotation_name_side = "right",
                           border = T
                           #annotation_legend_param = list(EdoDx =list(labels = "Edo Dx"),
                           #nrow = 3, legend.cex=0.7)
)

#-------------------Anotaciones filas---------------------#
VirulenceFactors <- rep(c("Adhesins", "Toxin", "Iron acquisition", "Protectins", "Island of pathogenicity"), c(10, 6, 6, 4, 1))

RAnn<-rowAnnotation("Virulence factors" = VirulenceFactors,
                    col=list("Virulence factors"=c("Adhesins"="#0495CF", "Toxin"="#01B964", "Iron acquisition"="#FB8F00",
                                                   "Protectins"="#D9276E", "Island of pathogenicity"="#BD64B4")),
                    annotation_height = unit(c(5,7,2,2,10,8), "mm"),
                    gp = gpar(col = "white"),
                    annotation_name_side = "top",
                    border=T)


rowanotacion = HeatmapAnnotation(X  = anno_barplot(data.frame(rowSums(matriz))[,1], gp = gpar(fill = "blue")),
                                 foo = anno_text(Serotipo, gp = gpar(fontsize = 9)), show_annotation_name = c(X = FALSE)
                                 )
rowanotacion2 = rowAnnotation(X  = anno_barplot(data.frame(colSums(matriz))[,1], gp = gpar(fill = "blue")),
                              show_annotation_name = c(X = FALSE))

#-------------------Heatmap gower distance---------------------#
table(gower.groups$cutree.fit.gower..k...6.)
HMBact <- Heatmap(t(matriz),
        col = col_fun,
        rect_gp = gpar(col = "white", lwd = 1.5),
        #name = "Proportion",
        #cluster_rows = FALSE,
        column_dend_height = unit(3, "cm"),
        cluster_columns = fit.gower,
        #        cluster_rows = fit2,
        column_names_gp = gpar(fontsize=7),
        show_heatmap_legend = F,
        row_names_side = "left", 
        column_names_side = "top",
        #cell_fun = function(j, i, x, y, w, h, col) { # add text to each grid
        #  grid.text(round(Firmas[i, j],2), x, y)
        #}, 
        left_annotation = RAnn, 
        right_annotation = rowanotacion2,
        bottom_annotation =  rowanotacion,
        #clustering_distance_columns = "manhattan",  #"euclidean", "maximum", "manhattan", "canberra", "binary" or "minkowski
        #clustering_method_columns = "median", #"ward.D", "ward.D2", "single", "complete", "average" (= UPGMA), "mcquitty" (= WPGMA), "median" (= WPGMC) or "centroid" (= UPGMC).
        top_annotation  = coann,
        #top_annotation = CAF,
        column_title = "Gower distance & Center clustering method",
        na_col = "#ABC4F0",
        border=T,
        row_names_gp = gpar(fontface="italic"
                            #,col=c(rep("#FF0A1C",13),rep("#03585D",9))
        ),
        column_split = 3#rep(c("A","B", "C", "D", "E", "F"), c(10, 20, 8, 42, 7, 13))
)

draw(HMBact, merge_legend = TRUE, heatmap_legend_side = "bottom", annotation_legend_side = "bottom")
#PDF 15 X 10 

draw(HMBact,
     heatmap_legend_side = "bottom", 
     annotation_legend_side = "bottom"#, 
     #heatmap_legend_param = list(direction = "horizontal")
)

#-------------------Heatmap euclidian distance---------------------#
HM2 <- Heatmap(t(matriz),
        col = col_fun,
        rect_gp = gpar(col = "white", lwd = 1.5),
        #name = "Proportion",
        #cluster_rows = FALSE,
        column_dend_height = unit(3, "cm"),
        cluster_columns = fit.euclidian,
        #        cluster_rows = fit2,
        column_names_gp = gpar(fontsize=7),
        show_heatmap_legend = F,
        row_names_side = "left", 
        column_names_side = "top",
        #cell_fun = function(j, i, x, y, w, h, col) { # add text to each grid
        #  grid.text(round(Firmas[i, j],2), x, y)
        #}, 
        left_annotation = RAnn, 
        right_annotation = rowanotacion2,
        bottom_annotation =  rowanotacion,
        #clustering_distance_columns = "manhattan",  #"euclidean", "maximum", "manhattan", "canberra", "binary" or "minkowski
        #clustering_method_columns = "median", #"ward.D", "ward.D2", "single", "complete", "average" (= UPGMA), "mcquitty" (= WPGMA), "median" (= WPGMC) or "centroid" (= UPGMC).
        top_annotation  = coann,
        #top_annotation = CAF,
        column_title = "Euclidean distance & Center clustering method",
        na_col = "#ABC4F0",
        border=T,
        row_names_gp = gpar(fontface="italic"
                            #,col=c(rep("#FF0A1C",13),rep("#03585D",9))
        ),
        column_split = 3#rep(c("A","B", "C", "D", "E", "F"), c(10, 20, 8, 42, 7, 13))
)

draw(HM2, merge_legend = TRUE, heatmap_legend_side = "bottom", annotation_legend_side = "bottom")
